from setuptools import setup, find_packages

setup(
    name="leg2",
    version="1.0",
    author="mark-cyber-cmd",
    author_email="dingyiming19991203@163.com",
    description="Learn to Pack Python Module  -->公众号：等离子工作室",

    # 项目主页
    url="https://www.baidu.com",

    # 你要安装的包，通过 setuptools.find_packages 找到当前目录下有哪些包
    packages=find_packages()
)
